<?php
namespace nur\sery\schema\types;

use nur\sery\schema\_scalar\ScalarResult;
use nur\sery\schema\_scalar\ScalarSchema;
use nur\sery\schema\input\Input;
use nur\sery\schema\Result;
use nur\sery\schema\Schema;

class tint implements IType {
  static function ensure_int(&$int): void {
    if (!is_int($int)) $int = intval($int);
  }

  static function ensure_nint(&$int): void {
    if ($int !== null) self::ensure_int($int);
  }

  function isAvailable(Input $input, $destKey): bool {
    return $input->isAvailable($destKey) && $input->get($destKey) !== false;
  }

  function isNull($value): bool {
    return $value === null || (is_string($value) && trim($value) === "");
  }

  const INT_PATTERN = '/^[-+]?[0-9]+(?:\.[0-9]*)?$/';

  function isValid($value, ?bool &$normalized=null): bool {
    if (is_string($value)) $valid = is_numeric(trim($value));
    else $valid = is_scalar($value);
    $normalized = is_int($value);
    return $valid;
  }

  /**
   * @var ScalarResult $result
   * @var ScalarSchema $schema
   */
  function verifix(&$value, Result &$result, Schema $schema): bool {
    if (is_int($value)) {
      $result->setNormalized();
      return false;
    } elseif (is_string($value)) {
      $int = trim($value);
      if (is_numeric($int)) $value = intval($int);
      else return $result->setInvalid($value, $schema);
    } elseif (is_scalar($value)) {
      $value = intval($value);
    } else {
      return $result->setInvalid($value, $schema);
    }
    $result->setValid();
    return true;
  }

  function format($value, $format=null): string {
    if ($format !== null) return sprintf($format, $value);
    else return strval($value);
  }
}
